# frozen_string_literal: true

using Corindon::DependencyInjection::Injectable

module Ekylibre
  module MultiTenancy
    class TenantRepository
      injectable(
        private_root: MultiTenancyPlugin::PRIVATE_ROOT,
        tenants_file: MultiTenancyPlugin::TENANTS_FILE,
      )

      # @param [Pathname] private_root
      def initialize(private_root:, tenants_file:)
        @private_root = private_root
        @tenants_file = tenants_file
        @write_mutex = Mutex.new
      end

      # @param [String] name
      # @return [Tenant, nil]
      def get(name)
        if has?(name)
          Tenant.new(name: name, private_directory: private_root.join(name))
        else
          nil
        end
      end

      def delete(name)
        tenants.delete(name)

        write
      end

      # @param [String] name
      # @return [Boolean]
      def has?(name)
        list.include?(name)
      end

      # @return [Array<String>]
      def list
        tenants.fetch(::Rails.env.to_s, [])
      end

      private

        # @return [Hash{String => Array<String>}]
        def load
          if tenants_file.exist?
            YAML.load_file(tenants_file) || {}
          else
            {}
          end
        end

        def write
          write_mutex.synchronize do
            tenants_file.write(tenants.to_yaml)
          end

          nil
        end

        # @return [Hash{String => Array<String>}]
        def tenants
          @tenants ||= load
        end

        # @return [Pathname]
        attr_reader :private_root
        # @return [Pathname]
        attr_reader :tenants_file
        # @return [Mutex]
        attr_reader :write_mutex
    end
  end
end
